# Copyright 2012 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc

import six

from keystone import exception


@six.add_metaclass(abc.ABCMeta)
class FederationDriverBase(object):

    @abc.abstractmethod
    def create_idp(self, idp_id, idp):
        """Create an identity provider.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :param idp: idp object
        :type idp: dict
        :returns: idp ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def delete_idp(self, idp_id):
        """Delete an identity provider.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_idp(self, idp_id):
        """Get an identity provider by ID.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :returns: idp ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_idp_from_remote_id(self, remote_id):
        """Get an identity provider by remote ID.

        :param remote_id: ID of remote IdP
        :type idp_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :returns: idp ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def update_idp(self, idp_id, idp):
        """Update an identity provider by ID.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :param idp: idp object
        :type idp: dict
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :returns: idp ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def create_protocol(self, idp_id, protocol_id, protocol):
        """Add an IdP-Protocol configuration.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :param protocol_id: ID of protocol object
        :type protocol_id: string
        :param protocol: protocol object
        :type protocol: dict
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :returns: protocol ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def update_protocol(self, idp_id, protocol_id, protocol):
        """Change an IdP-Protocol configuration.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :param protocol_id: ID of protocol object
        :type protocol_id: string
        :param protocol: protocol object
        :type protocol: dict
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :raises keystone.exception.FederatedProtocolNotFound: If the federated
            protocol cannot be found.
        :returns: protocol ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_protocol(self, idp_id, protocol_id):
        """Get an IdP-Protocol configuration.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :param protocol_id: ID of protocol object
        :type protocol_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :raises keystone.exception.FederatedProtocolNotFound: If the federated
            protocol cannot be found.
        :returns: protocol ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def list_protocols(self, idp_id):
        """List an IdP's supported protocols.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :returns: list of protocol ref
        :rtype: list of dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def delete_protocol(self, idp_id, protocol_id):
        """Delete an IdP-Protocol configuration.

        :param idp_id: ID of IdP object
        :type idp_id: string
        :param protocol_id: ID of protocol object
        :type protocol_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :raises keystone.exception.FederatedProtocolNotFound: If the federated
            protocol cannot be found.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def create_mapping(self, mapping_id, mapping):
        """Create a mapping.

        :param mapping_id: ID of mapping object
        :type mapping_id: string
        :param mapping: mapping ref with mapping name
        :type mapping: dict
        :returns: mapping ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def delete_mapping(self, mapping_id):
        """Delete a mapping.

        :param mapping_id: id of mapping to delete
        :type mapping_ref: string
        :returns: None

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def update_mapping(self, mapping_id, mapping_ref):
        """Update a mapping.

        :param mapping_id: id of mapping to update
        :type mapping_id: string
        :param mapping_ref: new mapping ref
        :type mapping_ref: dict
        :returns: mapping ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def list_mappings(self):
        """List all mappings.

        :returns: list of mapping refs
        :rtype: list of dicts

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_mapping(self, mapping_id):
        """Get a mapping, returns the mapping based on mapping_id.

        :param mapping_id: id of mapping to get
        :type mapping_ref: string
        :raises keystone.exception.MappingNotFound: If the mapping cannot
            be found.
        :returns: mapping ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_mapping_from_idp_and_protocol(self, idp_id, protocol_id):
        """Get mapping based on idp_id and protocol_id.

        :param idp_id: id of the identity provider
        :type idp_id: string
        :param protocol_id: id of the protocol
        :type protocol_id: string
        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.
        :raises keystone.exception.FederatedProtocolNotFound: If the federated
            protocol cannot be found.
        :returns: mapping ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def create_sp(self, sp_id, sp):
        """Create a service provider.

        :param sp_id: id of the service provider
        :type sp_id: string
        :param sp: service provider object
        :type sp: dict

        :returns: service provider ref
        :rtype: dict

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def delete_sp(self, sp_id):
        """Delete a service provider.

        :param sp_id: id of the service provider
        :type sp_id: string

        :raises keystone.exception.ServiceProviderNotFound: If the service
            provider doesn't exist.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_sp(self, sp_id):
        """Get a service provider.

        :param sp_id: id of the service provider
        :type sp_id: string
        :returns: service provider ref
        :rtype: dict

        :raises keystone.exception.ServiceProviderNotFound: If the service
            provider doesn't exist.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def update_sp(self, sp_id, sp):
        """Update a service provider.

        :param sp_id: id of the service provider
        :type sp_id: string
        :param sp: service prvider object
        :type sp: dict

        :returns: service provider ref
        :rtype: dict

        :raises keystone.exception.ServiceProviderNotFound: If the service
            provider doesn't exist.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def get_enabled_service_providers(self):
        """List enabled service providers for Service Catalog.

        Service Provider in a catalog contains three attributes: ``id``,
        ``auth_url``, ``sp_url``, where:

        - id is a unique, user defined identifier for service provider object
        - auth_url is an authentication URL of remote Keystone
        - sp_url a URL accessible at the remote service provider where SAML
          assertion is transmitted.

        :returns: list of dictionaries with enabled service providers
        :rtype: list of dicts

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def list_idps(self, hints):
        """List all identity providers.

        :param hints: filter hints which the driver should
                      implement if at all possible.
        :returns: list of idp refs
        :rtype: list of dicts

        :raises keystone.exception.IdentityProviderNotFound: If the IdP
            doesn't exist.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def list_sps(self, hints):
        """List all service providers.

        :param hints: filter hints which the driver should
                      implement if at all possible.
        :returns: List of service provider ref objects
        :rtype: list of dicts

        :raises keystone.exception.ServiceProviderNotFound: If the SP
            doesn't exist.

        """
        raise exception.NotImplemented()  # pragma: no cover



# Copyright 2019 SUSE Linux GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import migrate
import sqlalchemy as sql


def upgrade(migrate_engine):
    meta = sql.MetaData()
    meta.bind = migrate_engine

    access_rule = sql.Table('access_rule', meta, autoload=True)

    external_id = sql.Column('external_id', sql.String(64))
    access_rule.create_column(external_id)
    sql.Index('external_id', access_rule.c.external_id).create()
    unique_constraint_id = migrate.UniqueConstraint('external_id',
                                                    table=access_rule)
    unique_constraint_id.create()

    user_id = sql.Column('user_id', sql.String(64))
    access_rule.create_column(user_id)
    sql.Index('user_id', access_rule.c.user_id).create()
    unique_constraint_rule_for_user = migrate.UniqueConstraint(
        'user_id', 'service', 'path', 'method',
        name='duplicate_access_rule_for_user_constraint',
        table=access_rule)
    unique_constraint_rule_for_user.create()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ldap


from keystone.common import cache
from keystone.common import provider_api
import keystone.conf
from keystone.tests import unit
from keystone.tests.unit import default_fixtures
from keystone.tests.unit.ksfixtures import database
from keystone.tests.unit.ksfixtures import ldapdb


CONF = keystone.conf.CONF
PROVIDERS = provider_api.ProviderAPIs


def create_group_container(identity_api):
    # Create the groups base entry (ou=Groups,cn=example,cn=com)
    group_api = identity_api.driver.group
    conn = group_api.get_connection()
    dn = 'ou=Groups,cn=example,cn=com'
    conn.add_s(dn, [('objectclass', ['organizationalUnit']),
                    ('ou', ['Groups'])])


class BaseBackendLdapCommon(object):
    """Mixin class to set up generic LDAP backends."""

    def setUp(self):
        super(BaseBackendLdapCommon, self).setUp()
        self.useFixture(ldapdb.LDAPDatabase())

        self.load_backends()
        self.load_fixtures(default_fixtures)

    def _get_domain_fixture(self):
        """Return the static domain, since domains in LDAP are read-only."""
        return PROVIDERS.resource_api.get_domain(
            CONF.identity.default_domain_id
        )

    def get_config(self, domain_id):
        # Only one conf structure unless we are using separate domain backends
        return CONF

    def config_overrides(self):
        super(BaseBackendLdapCommon, self).config_overrides()
        self.config_fixture.config(group='identity', driver='ldap')

    def config_files(self):
        config_files = super(BaseBackendLdapCommon, self).config_files()
        config_files.append(unit.dirs.tests_conf('backend_ldap.conf'))
        return config_files

    def get_user_enabled_vals(self, user):
        user_dn = (
            PROVIDERS.identity_api.driver.user._id_to_dn_string(
                user['id']
            )
        )
        enabled_attr_name = CONF.ldap.user_enabled_attribute

        ldap_ = PROVIDERS.identity_api.driver.user.get_connection()
        res = ldap_.search_s(user_dn,
                             ldap.SCOPE_BASE,
                             u'(sn=%s)' % user['name'])
        if enabled_attr_name in res[0][1]:
            return res[0][1][enabled_attr_name]
        else:
            return None


class BaseBackendLdap(object):
    """Mixin class to set up an all-LDAP configuration."""

    def setUp(self):
        # NOTE(dstanek): The database must be setup prior to calling the
        # parent's setUp. The parent's setUp uses services (like
        # credentials) that require a database.
        self.useFixture(database.Database())
        super(BaseBackendLdap, self).setUp()

    def load_fixtures(self, fixtures):
        # Override super impl since need to create group container.
        create_group_container(PROVIDERS.identity_api)
        super(BaseBackendLdap, self).load_fixtures(fixtures)


class BaseBackendLdapIdentitySqlEverythingElse(unit.SQLDriverOverrides):
    """Mixin base for Identity LDAP, everything else SQL backend tests."""

    def config_files(self):
        config_files = super(BaseBackendLdapIdentitySqlEverythingElse,
                             self).config_files()
        config_files.append(unit.dirs.tests_conf('backend_ldap_sql.conf'))
        return config_files

    def setUp(self):
        sqldb = self.useFixture(database.Database())
        super(BaseBackendLdapIdentitySqlEverythingElse, self).setUp()
        self.load_backends()
        cache.configure_cache()

        sqldb.recreate()
        self.load_fixtures(default_fixtures)
        # defaulted by the data load
        self.user_foo['enabled'] = True

    def config_overrides(self):
        super(BaseBackendLdapIdentitySqlEverythingElse,
              self).config_overrides()
        self.config_fixture.config(group='identity', driver='ldap')
        self.config_fixture.config(group='resource', driver='sql')
        self.config_fixture.config(group='assignment', driver='sql')


class BaseBackendLdapIdentitySqlEverythingElseWithMapping(object):
    """Mixin base class to test mapping of default LDAP backend.

    The default configuration is not to enable mapping when using a single
    backend LDAP driver.  However, a cloud provider might want to enable
    the mapping, hence hiding the LDAP IDs from any clients of keystone.
    Setting backward_compatible_ids to False will enable this mapping.

    """

    def config_overrides(self):
        super(BaseBackendLdapIdentitySqlEverythingElseWithMapping,
              self).config_overrides()
        self.config_fixture.config(group='identity_mapping',
                                   backward_compatible_ids=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sql

from keystone.common.sql import upgrades


def upgrade(migrate_engine):
    meta = sql.MetaData()
    meta.bind = migrate_engine

    idp_table = sql.Table('identity_provider', meta, autoload=True)
    idp_table.c.domain_id.alter(nullable=False, unique=True)

    if upgrades.USE_TRIGGERS:
        if migrate_engine.name == 'postgresql':
            drop_idp_insert_trigger = (
                'DROP TRIGGER idp_insert_read_only on identity_provider;'
            )
        elif migrate_engine.name == 'mysql':
            drop_idp_insert_trigger = (
                'DROP TRIGGER idp_insert_read_only;'
            )
        else:
            drop_idp_insert_trigger = (
                'DROP TRIGGER IF EXISTS idp_insert_read_only;'
            )
        migrate_engine.execute(drop_idp_insert_trigger)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy as sql
import sqlalchemy.sql.expression as expression


def upgrade(migrate_engine):
    meta = sql.MetaData()
    meta.bind = migrate_engine

    password_table = sql.Table('password', meta, autoload=True)
    with migrate_engine.begin() as conn:
        stmt = password_table.update().where(
            password_table.c.password_hash == expression.null()).values(
            {'password_hash': password_table.c.password})
        conn.execute(stmt)



# Copyright 2012 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
import datetime

from oslo_utils import timeutils
import six

import keystone.conf
from keystone import exception


CONF = keystone.conf.CONF


def revoked_before_cutoff_time():
    expire_delta = datetime.timedelta(
        seconds=CONF.token.expiration + CONF.revoke.expiration_buffer)
    oldest = timeutils.utcnow() - expire_delta
    return oldest


@six.add_metaclass(abc.ABCMeta)
class RevokeDriverBase(object):
    """Interface for recording and reporting revocation events."""

    @abc.abstractmethod
    def list_events(self, last_fetch=None, token=None):
        """return the revocation events, as a list of objects.

        :param last_fetch:   Time of last fetch.  Return all events newer.
        :param token: dictionary of values from a token, normalized for
                      differences between v2 and v3. The checked values are a
                      subset of the attributes of model.TokenEvent
        :returns: A list of keystone.revoke.model.RevokeEvent
                  newer than `last_fetch.`
                  If no last_fetch is specified, returns all events
                  for tokens issued after the expiration cutoff.

        """
        raise exception.NotImplemented()  # pragma: no cover

    @abc.abstractmethod
    def revoke(self, event):
        """register a revocation event.

        :param event: An instance of
            keystone.revoke.model.RevocationEvent

        """
        raise exception.NotImplemented()  # pragma: no cover



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from oslo_serialization import jsonutils
from six.moves import http_client

from keystone.common import provider_api
import keystone.conf
from keystone.tests.common import auth as common_auth
from keystone.tests import unit
from keystone.tests.unit import base_classes
from keystone.tests.unit import ksfixtures
from keystone.tests.unit.ksfixtures import temporaryfile

CONF = keystone.conf.CONF
PROVIDERS = provider_api.ProviderAPIs


class TrustTests(base_classes.TestCaseWithBootstrap,
                 common_auth.AuthTestMixin):
    """Common functionality for all trust tests.

    Sets up trustor and trustee users and trust.
    """

    def setUp(self):
        super(TrustTests, self).setUp()
        self.loadapp()
        self.policy_file = self.useFixture(temporaryfile.SecureTempFile())
        self.policy_file_name = self.policy_file.file_name
        self.useFixture(
            ksfixtures.Policy(
                self.config_fixture, policy_file=self.policy_file_name
            )
        )

        domain = PROVIDERS.resource_api.create_domain(
            uuid.uuid4().hex, unit.new_domain_ref()
        )
        self.domain_id = domain['id']

        trustor_user = unit.new_user_ref(domain_id=self.domain_id)
        self.trustor_user_id = PROVIDERS.identity_api.create_user(
            trustor_user)['id']
        trustee_user = unit.new_user_ref(domain_id=self.domain_id)
        self.trustee_user_id = PROVIDERS.identity_api.create_user(
            trustee_user)['id']
        project = PROVIDERS.resource_api.create_project(
            uuid.uuid4().hex, unit.new_project_ref(domain_id=self.domain_id)
        )
        self.project_id = project['id']
        PROVIDERS.assignment_api.create_grant(
            self.bootstrapper.member_role_id, user_id=self.trustor_user_id,
            project_id=self.project_id
        )
        PROVIDERS.assignment_api.create_grant(
            self.bootstrapper.member_role_id, user_id=self.trustee_user_id,
            project_id=project['id']
        )
        self.trust_id = uuid.uuid4().hex
        self.trust_data = {
            'trust': {'trustor_user_id': self.trustor_user_id,
                      'trustee_user_id': self.trustee_user_id,
                      'project_id': self.project_id,
                      'impersonation': False},
            'roles': [{"id": self.bootstrapper.member_role_id}]
        }
        auth = self.build_authentication_request(
            user_id=self.trustor_user_id,
            password=trustor_user['password'],
            project_id=project['id']
        )
        # Grab a token using the trustor persona we're testing and prepare
        # headers for requests we'll be making in the tests.
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.trustor_headers = {'X-Auth-Token': self.token_id}

        auth = self.build_authentication_request(
            user_id=self.trustee_user_id,
            password=trustee_user['password'],
            project_id=project['id']
        )
        # Grab a token using the trustee persona we're testing and prepare
        # headers for requests we'll be making in the tests.
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.trustee_headers = {'X-Auth-Token': self.token_id}

    def _override_policy_old_defaults(self):
        # TODO(cmurphy): This is to simulate what would happen if the operator
        # had generated a sample policy config, or had never removed their old
        # policy files since we adopted policy in code, and had explicitly
        # retained the old "" policy check strings. Remove this once the
        # hardcoded enforcement is removed from the trusts API.
        with open(self.policy_file_name, 'w') as f:
            overridden_policies = {
                'identity:list_trusts': '',
                'identity:delete_trust': '',
                'identity:get_trust': '',
                'identity:list_roles_for_trust': '',
                'identity:get_role_for_trust': '',
            }
            f.write(jsonutils.dumps(overridden_policies))


class _AdminTestsMixin(object):
    """Tests for all admin users.

    This exercises both the is_admin user and users granted the admin role on
    the system scope.
    """

    def test_admin_cannot_create_trust_for_other_user(self):
        json = {'trust': self.trust_data['trust']}
        json['trust']['roles'] = self.trust_data['roles']

        with self.test_client() as c:
            c.post(
                '/v3/OS-TRUST/trusts',
                json=json,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_admin_list_all_trusts(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts',
                headers=self.headers
            )
        self.assertEqual(1, len(r.json['trusts']))


class AdminTokenTests(TrustTests, _AdminTestsMixin):
    """Tests for the is_admin user.

    The Trusts API has hardcoded is_admin checks that we need to ensure are
    preserved through the system-scope transition.
    """

    def setUp(self):
        super(AdminTokenTests, self).setUp()
        self.config_fixture.config(admin_token='ADMIN')
        self.headers = {'X-Auth-Token': 'ADMIN'}

    def test_admin_can_delete_trust_for_other_user(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.headers,
                expected_status_code=http_client.NO_CONTENT
            )

    def test_admin_can_get_non_existent_trust_not_found(self):
        trust_id = uuid.uuid4().hex
        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % trust_id,
                headers=self.headers,
                expected_status_code=http_client.NOT_FOUND
            )

    def test_admin_cannot_get_trust_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % self.trust_id,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_admin_cannot_list_trust_roles_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_admin_cannot_get_trust_role_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )


class _SystemUserTests(object):
    """Tests for system admin, member, and reader."""

    def test_user_can_get_non_existent_trust(self):
        trust_id = uuid.uuid4().hex
        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % trust_id,
                headers=self.headers,
                expected_status_code=http_client.NOT_FOUND
            )

    def test_user_can_get_trust_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s' % self.trust_id,
                headers=self.headers
            )
        self.assertEqual(r.json['trust']['id'], self.trust_id)

    def test_user_can_list_trusts_for_trustee(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.headers
            )

    def test_user_can_list_trusts_for_trustor(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.headers
            )

    def test_user_can_list_trust_roles_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.headers
            )
        self.assertEqual(r.json['roles'][0]['id'],
                         self.bootstrapper.member_role_id)

    def test_user_can_get_trust_role_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.headers
            )


class _SystemReaderMemberTests(_SystemUserTests):
    """Tests for system readers and members."""

    def test_user_cannot_create_trust(self):
        json = {'trust': self.trust_data['trust']}
        json['trust']['roles'] = self.trust_data['roles']

        with self.test_client() as c:
            c.post(
                '/v3/OS-TRUST/trusts',
                json=json,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_delete_trust(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )


class SystemReaderTests(TrustTests, _SystemReaderMemberTests):
    """Tests for system reader users."""

    def setUp(self):
        super(SystemReaderTests, self).setUp()
        self.config_fixture.config(group='oslo_policy', enforce_scope=True)

        system_reader = unit.new_user_ref(
            domain_id=CONF.identity.default_domain_id
        )
        self.user_id = PROVIDERS.identity_api.create_user(
            system_reader
        )['id']
        PROVIDERS.assignment_api.create_system_grant_for_user(
            self.user_id, self.bootstrapper.reader_role_id
        )

        auth = self.build_authentication_request(
            user_id=self.user_id,
            password=system_reader['password'],
            system=True
        )

        # Grab a token using the persona we're testing and prepare headers
        # for requests we'll be making in the tests.
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.headers = {'X-Auth-Token': self.token_id}


class SystemMemberTests(TrustTests, _SystemReaderMemberTests):
    """Tests for system member users."""

    def setUp(self):
        super(SystemMemberTests, self).setUp()
        self.config_fixture.config(group='oslo_policy', enforce_scope=True)

        system_member = unit.new_user_ref(
            domain_id=CONF.identity.default_domain_id
        )
        self.user_id = PROVIDERS.identity_api.create_user(
            system_member
        )['id']
        PROVIDERS.assignment_api.create_system_grant_for_user(
            self.user_id, self.bootstrapper.member_role_id
        )

        auth = self.build_authentication_request(
            user_id=self.user_id,
            password=system_member['password'],
            system=True
        )

        # Grab a token using the persona we're testing and prepare headers
        # for requests we'll be making in the tests.
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.headers = {'X-Auth-Token': self.token_id}


class SystemAdminTests(TrustTests, _AdminTestsMixin, _SystemUserTests):
    """Tests for system admin users."""

    def setUp(self):
        super(SystemAdminTests, self).setUp()
        self.config_fixture.config(group='oslo_policy', enforce_scope=True)

        self.user_id = self.bootstrapper.admin_user_id
        auth = self.build_authentication_request(
            user_id=self.user_id,
            password=self.bootstrapper.admin_password,
            system=True
        )

        # Grab a token using the persona we're testing and prepare headers
        # for requests we'll be making in the tests.
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.headers = {'X-Auth-Token': self.token_id}

    def test_admin_can_delete_trust_for_other_user(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.headers
            )

    def test_admin_cannot_delete_trust_for_user_overridden_defaults(self):
        # only the is_admin admin can do this
        self._override_policy_old_defaults()
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_admin_cannot_get_trust_for_other_user_overridden_defaults(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % self.trust_id,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_admin_cannot_list_roles_for_other_user_overridden_defaults(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_admin_cannot_get_trust_role_for_other_user_overridden_defaults(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_list_all_trusts_overridden_defaults(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts',
                headers=self.headers
            )
        self.assertEqual(1, len(r.json['trusts']))


class ProjectUserTests(TrustTests):
    """Tests for all project users."""

    def setUp(self):
        super(ProjectUserTests, self).setUp()
        other_user = unit.new_user_ref(domain_id=self.domain_id)
        self.other_user_id = PROVIDERS.identity_api.create_user(
            other_user)['id']
        PROVIDERS.assignment_api.create_grant(
            self.bootstrapper.member_role_id, user_id=self.other_user_id,
            project_id=self.project_id
        )

        auth = self.build_authentication_request(
            user_id=self.other_user_id,
            password=other_user['password'],
            project_id=self.project_id
        )
        # Grab a token using another persona who has no trusts associated with
        # them
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.other_headers = {'X-Auth-Token': self.token_id}

    def test_user_can_list_trusts_of_whom_they_are_the_trustor(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.trustor_headers
            )
        self.assertEqual(1, len(r.json['trusts']))
        self.assertEqual(self.trust_id, r.json['trusts'][0]['id'])

    def test_user_can_list_trusts_delegated_to_them(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.trustee_headers
            )
        self.assertEqual(1, len(r.json['trusts']))
        self.assertEqual(self.trust_id, r.json['trusts'][0]['id'])

    def test_trustor_cannot_list_trusts_for_trustee(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.trustor_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustee_cannot_list_trusts_for_trustor(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_trusts_for_other_trustor(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_trusts_for_other_trustee(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_all_trusts(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts',
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_get_another_users_trust(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_can_get_non_existent_trust_not_found(self):
        trust_id = uuid.uuid4().hex
        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % trust_id,
                headers=self.other_headers,
                expected_status_code=http_client.NOT_FOUND
            )

    def test_user_can_get_trust_of_whom_they_are_the_trustor(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustor_headers
            )

    def test_user_can_get_trust_delegated_to_them(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustee_headers
            )
        self.assertEqual(r.json['trust']['id'], self.trust_id)

    def test_trustor_can_create_trust(self):
        json = {'trust': self.trust_data['trust']}
        json['trust']['roles'] = self.trust_data['roles']

        with self.test_client() as c:
            c.post(
                '/v3/OS-TRUST/trusts',
                json=json,
                headers=self.trustor_headers
            )

    def test_trustee_cannot_create_trust(self):
        json = {'trust': self.trust_data['trust']}
        json['trust']['roles'] = self.trust_data['roles']

        with self.test_client() as c:
            c.post(
                '/v3/OS-TRUST/trusts',
                json=json,
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustor_can_delete_trust(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustor_headers
            )

    def test_trustee_cannot_delete_trust(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_delete_trust_for_other_user(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustor_can_list_trust_roles(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.trustor_headers
            )
        self.assertEqual(r.json['roles'][0]['id'],
                         self.bootstrapper.member_role_id)

    def test_trustee_can_list_trust_roles(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.trustee_headers
            )
        self.assertEqual(r.json['roles'][0]['id'],
                         self.bootstrapper.member_role_id)

    def test_user_cannot_list_trust_roles_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustor_can_get_trust_role(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.trustor_headers
            )

    def test_trustee_can_get_trust_role(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.trustee_headers
            )

    def test_user_cannot_get_trust_role_for_other_user(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustor_cannot_list_trusts_for_trustee_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.trustor_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustee_cannot_list_trusts_for_trustor_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_trusts_for_other_trustor_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_trusts_for_trustee_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_all_trusts_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts',
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustor_can_delete_trust_overridden_default(self):
        self._override_policy_old_defaults()
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustor_headers
            )

    def test_trustee_cannot_delete_trust_overridden_default(self):
        self._override_policy_old_defaults()
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustee_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_delete_trust_for_other_user_overridden_default(self):
        self._override_policy_old_defaults()
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_can_get_trust_of_whom_they_are_the_trustor_overridden_default(self):
        self._override_policy_old_defaults()
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustor_headers
            )

    def test_user_can_get_trust_delegated_to_them_overridden_default(self):
        self._override_policy_old_defaults()
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.trustee_headers
            )
        self.assertEqual(r.json['trust']['id'], self.trust_id)

    def test_trustor_can_list_trust_roles_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.trustor_headers
            )
        self.assertEqual(r.json['roles'][0]['id'],
                         self.bootstrapper.member_role_id)

    def test_trustee_can_list_trust_roles_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            r = c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.trustee_headers
            )
        self.assertEqual(r.json['roles'][0]['id'],
                         self.bootstrapper.member_role_id)

    def test_user_cannot_list_trust_roles_other_user_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustor_can_get_trust_role_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.trustor_headers
            )

    def test_trustee_can_get_trust_role_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.trustee_headers
            )

    def test_user_cannot_get_trust_role_other_user_overridden_default(self):
        self._override_policy_old_defaults()
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.other_headers,
                expected_status_code=http_client.FORBIDDEN
            )


class DomainUserTests(TrustTests):
    """Tests for all domain users.

    Domain users should not be able to interact with trusts at all.
    """

    def setUp(self):
        super(DomainUserTests, self).setUp()
        self.config_fixture.config(group='oslo_policy', enforce_scope=True)
        domain_admin = unit.new_user_ref(domain_id=self.domain_id)
        self.user_id = PROVIDERS.identity_api.create_user(
            domain_admin)['id']
        PROVIDERS.assignment_api.create_grant(
            self.bootstrapper.admin_role_id, user_id=self.user_id,
            domain_id=self.domain_id
        )

        auth = self.build_authentication_request(
            user_id=self.user_id,
            password=domain_admin['password'],
            domain_id=self.domain_id
        )
        # Grab a token using another persona who has no trusts associated with
        # them
        with self.test_client() as c:
            r = c.post('/v3/auth/tokens', json=auth)
            self.token_id = r.headers['X-Subject-Token']
            self.headers = {'X-Auth-Token': self.token_id}

    def test_trustor_cannot_list_trusts_for_trustee(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustee_user_id=%s' %
                 self.trustee_user_id),
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_trustee_cannot_list_trusts_for_trustor(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                ('/v3/OS-TRUST/trusts?trustor_user_id=%s' %
                 self.trustor_user_id),
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_all_trusts(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts',
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_get_trust(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_can_get_non_existent_trust_not_found(self):
        trust_id = uuid.uuid4().hex
        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s' % trust_id,
                headers=self.headers,
                expected_status_code=http_client.NOT_FOUND
            )

    def test_user_cannot_create_trust(self):
        trust_data = self.trust_data['trust']
        trust_data['trustor_user_id'] = self.user_id
        json = {'trust': trust_data}
        json['trust']['roles'] = self.trust_data['roles']

        with self.test_client() as c:
            c.post(
                '/v3/OS-TRUST/trusts',
                json=json,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_delete_trust(self):
        ref = PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.delete(
                '/v3/OS-TRUST/trusts/%s' % ref['id'],
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_list_trust_roles(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.get(
                '/v3/OS-TRUST/trusts/%s/roles' % self.trust_id,
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )

    def test_user_cannot_get_trust_role(self):
        PROVIDERS.trust_api.create_trust(
            self.trust_id, **self.trust_data)

        with self.test_client() as c:
            c.head(
                ('/v3/OS-TRUST/trusts/%s/roles/%s' %
                 (self.trust_id, self.bootstrapper.member_role_id)),
                headers=self.headers,
                expected_status_code=http_client.FORBIDDEN
            )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import versionutils
from oslo_policy import policy

from keystone.common.policies import base

SYSTEM_READER_OR_DOMAIN_READER = (
    '(' + base.SYSTEM_READER + ') or '
    '(role:reader and domain_id:%(target.domain_id)s)'
)
SYSTEM_READER_OR_PROJECT_DOMAIN_READER_OR_PROJECT_ADMIN = (
    '(' + base.SYSTEM_READER + ') or '
    '(role:reader and domain_id:%(target.project.domain_id)s) or '
    '(role:admin and project_id:%(target.project.id)s)'
)

deprecated_list_role_assignments = policy.DeprecatedRule(
    name=base.IDENTITY % 'list_role_assignments',
    check_str=base.RULE_ADMIN_REQUIRED
)
deprecated_list_role_assignments_for_tree = policy.DeprecatedRule(
    name=base.IDENTITY % 'list_role_assignments_for_tree',
    check_str=base.RULE_ADMIN_REQUIRED
)

DEPRECATED_REASON = (
    "The assignment API is now aware of system scope and default roles."
)

role_assignment_policies = [
    policy.DocumentedRuleDefault(
        name=base.IDENTITY % 'list_role_assignments',
        check_str=SYSTEM_READER_OR_DOMAIN_READER,
        scope_types=['system', 'domain'],
        description='List role assignments.',
        operations=[{'path': '/v3/role_assignments',
                     'method': 'GET'},
                    {'path': '/v3/role_assignments',
                     'method': 'HEAD'}],
        deprecated_rule=deprecated_list_role_assignments,
        deprecated_reason=DEPRECATED_REASON,
        deprecated_since=versionutils.deprecated.STEIN),
    policy.DocumentedRuleDefault(
        name=base.IDENTITY % 'list_role_assignments_for_tree',
        check_str=SYSTEM_READER_OR_PROJECT_DOMAIN_READER_OR_PROJECT_ADMIN,
        scope_types=['system', 'domain', 'project'],
        description=('List all role assignments for a given tree of '
                     'hierarchical projects.'),
        operations=[{'path': '/v3/role_assignments?include_subtree',
                     'method': 'GET'},
                    {'path': '/v3/role_assignments?include_subtree',
                     'method': 'HEAD'}],
        deprecated_rule=deprecated_list_role_assignments_for_tree,
        deprecated_reason=DEPRECATED_REASON,
        deprecated_since=versionutils.deprecated.TRAIN),

]


def list_rules():
    return role_assignment_policies



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Keystone's pep8 extensions.

In order to make the review process faster and easier for core devs we are
adding some Keystone specific pep8 checks. This will catch common errors
so that core devs don't have to.

There are two types of pep8 extensions. One is a function that takes either
a physical or logical line. The physical or logical line is the first param
in the function definition and can be followed by other parameters supported
by pycodestyle. The second type is a class that parses AST trees. For more info
please see pycodestyle.py.
"""

import ast
import re

import six


class BaseASTChecker(ast.NodeVisitor):
    """Provides a simple framework for writing AST-based checks.

    Subclasses should implement visit_* methods like any other AST visitor
    implementation. When they detect an error for a particular node the
    method should call ``self.add_error(offending_node)``. Details about
    where in the code the error occurred will be pulled from the node
    object.

    Subclasses should also provide a class variable named CHECK_DESC to
    be used for the human readable error message.

    """

    def __init__(self, tree, filename):
        """Created object automatically by pep8.

        :param tree: an AST tree
        :param filename: name of the file being analyzed
                         (ignored by our checks)
        """
        self._tree = tree
        self._errors = []

    def run(self):
        """Called automatically by pep8."""
        self.visit(self._tree)
        return self._errors

    def add_error(self, node, message=None):
        """Add an error caused by a node to the list of errors for pep8."""
        message = message or self.CHECK_DESC
        error = (node.lineno, node.col_offset, message, self.__class__)
        self._errors.append(error)


class CheckForMutableDefaultArgs(BaseASTChecker):
    """Check for the use of mutable objects as function/method defaults.

    We are only checking for list and dict literals at this time. This means
    that a developer could specify an instance of their own and cause a bug.
    The fix for this is probably more work than it's worth because it will
    get caught during code review.

    """

    CHECK_DESC = 'K001 Using mutable as a function/method default'
    MUTABLES = (
        ast.List, ast.ListComp,
        ast.Dict, ast.DictComp,
        ast.Set, ast.SetComp,
        ast.Call)

    def visit_FunctionDef(self, node):
        for arg in node.args.defaults:
            if isinstance(arg, self.MUTABLES):
                self.add_error(arg)

        super(CheckForMutableDefaultArgs, self).generic_visit(node)


def block_comments_begin_with_a_space(physical_line, line_number):
    """There should be a space after the # of block comments.

    There is already a check in pep8 that enforces this rule for
    inline comments.

    Okay: # this is a comment
    Okay: #!/usr/bin/python
    Okay: #  this is a comment
    K002: #this is a comment

    """
    MESSAGE = "K002 block comments should start with '# '"

    # shebangs are OK
    if line_number == 1 and physical_line.startswith('#!'):
        return

    text = physical_line.strip()
    if text.startswith('#'):  # look for block comments
        if len(text) > 1 and not text[1].isspace():
            return physical_line.index('#'), MESSAGE


class CheckForTranslationIssues(BaseASTChecker):

    LOGGING_CHECK_DESC = 'K005 Using translated string in logging'
    USING_DEPRECATED_WARN = 'K009 Using the deprecated Logger.warn'
    LOG_MODULES = ('logging', 'oslo_log.log')
    I18N_MODULES = (
        'keystone.i18n._',
    )
    TRANS_HELPER_MAP = {
        'debug': None,
        'info': '_LI',
        'warning': '_LW',
        'error': '_LE',
        'exception': '_LE',
        'critical': '_LC',
    }

    def __init__(self, tree, filename):
        super(CheckForTranslationIssues, self).__init__(tree, filename)

        self.logger_names = []
        self.logger_module_names = []
        self.i18n_names = {}

        # NOTE(dstanek): this kinda accounts for scopes when talking
        # about only leaf node in the graph
        self.assignments = {}

    def generic_visit(self, node):
        """Called if no explicit visitor function exists for a node."""
        for field, value in ast.iter_fields(node):
            if isinstance(value, list):
                for item in value:
                    if isinstance(item, ast.AST):
                        item._parent = node
                        self.visit(item)
            elif isinstance(value, ast.AST):
                value._parent = node
                self.visit(value)

    def _filter_imports(self, module_name, alias):
        """Keep lists of logging and i18n imports."""
        if module_name in self.LOG_MODULES:
            self.logger_module_names.append(alias.asname or alias.name)
        elif module_name in self.I18N_MODULES:
            self.i18n_names[alias.asname or alias.name] = alias.name

    def visit_Import(self, node):
        for alias in node.names:
            self._filter_imports(alias.name, alias)
        return super(CheckForTranslationIssues, self).generic_visit(node)

    def visit_ImportFrom(self, node):
        for alias in node.names:
            full_name = '%s.%s' % (node.module, alias.name)
            self._filter_imports(full_name, alias)
        return super(CheckForTranslationIssues, self).generic_visit(node)

    def _find_name(self, node):
        """Return the fully qualified name or a Name or Attribute."""
        if isinstance(node, ast.Name):
            return node.id
        elif (isinstance(node, ast.Attribute)
                and isinstance(node.value, (ast.Name, ast.Attribute))):
            method_name = node.attr
            obj_name = self._find_name(node.value)
            if obj_name is None:
                return None
            return obj_name + '.' + method_name
        elif isinstance(node, six.string_types):
            return node
        else:  # could be Subscript, Call or many more
            return None

    def visit_Assign(self, node):
        """Look for 'LOG = logging.getLogger'.

        This handles the simple case:
          name = [logging_module].getLogger(...)

          - or -

          name = [i18n_name](...)

        And some much more comple ones:
          name = [i18n_name](...) % X

          - or -

          self.name = [i18n_name](...) % X

        """
        attr_node_types = (ast.Name, ast.Attribute)

        if (len(node.targets) != 1
                or not isinstance(node.targets[0], attr_node_types)):
            # say no to: "x, y = ..."
            return super(CheckForTranslationIssues, self).generic_visit(node)

        target_name = self._find_name(node.targets[0])

        if (isinstance(node.value, ast.BinOp) and
                isinstance(node.value.op, ast.Mod)):
            if (isinstance(node.value.left, ast.Call) and
                    isinstance(node.value.left.func, ast.Name) and
                    node.value.left.func.id in self.i18n_names):
                # NOTE(dstanek): this is done to match cases like:
                # `msg = _('something %s') % x`
                node = ast.Assign(value=node.value.left)

        if not isinstance(node.value, ast.Call):
            # node.value must be a call to getLogger
            self.assignments.pop(target_name, None)
            return super(CheckForTranslationIssues, self).generic_visit(node)

        # is this a call to an i18n function?
        if (isinstance(node.value.func, ast.Name)
                and node.value.func.id in self.i18n_names):
            self.assignments[target_name] = node.value.func.id
            return super(CheckForTranslationIssues, self).generic_visit(node)

        if (not isinstance(node.value.func, ast.Attribute)
                or not isinstance(node.value.func.value, attr_node_types)):
            # function must be an attribute on an object like
            # logging.getLogger
            return super(CheckForTranslationIssues, self).generic_visit(node)

        object_name = self._find_name(node.value.func.value)
        func_name = node.value.func.attr

        if (object_name in self.logger_module_names
                and func_name == 'getLogger'):
            self.logger_names.append(target_name)

        return super(CheckForTranslationIssues, self).generic_visit(node)

    def visit_Call(self, node):
        """Look for the 'LOG.*' calls."""
        # obj.method
        if isinstance(node.func, ast.Attribute):
            obj_name = self._find_name(node.func.value)
            if isinstance(node.func.value, ast.Name):
                method_name = node.func.attr
            elif isinstance(node.func.value, ast.Attribute):
                obj_name = self._find_name(node.func.value)
                method_name = node.func.attr
            else:  # could be Subscript, Call or many more
                return (super(CheckForTranslationIssues, self)
                        .generic_visit(node))

            # if dealing with a logger the method can't be "warn"
            if obj_name in self.logger_names and method_name == 'warn':
                msg = node.args[0]  # first arg to a logging method is the msg
                self.add_error(msg, message=self.USING_DEPRECATED_WARN)

            # must be a logger instance and one of the support logging methods
            if (obj_name not in self.logger_names
                    or method_name not in self.TRANS_HELPER_MAP):
                return (super(CheckForTranslationIssues, self)
                        .generic_visit(node))

            # the call must have arguments
            if not node.args:
                return (super(CheckForTranslationIssues, self)
                        .generic_visit(node))

            self._process_log_messages(node)

        return super(CheckForTranslationIssues, self).generic_visit(node)

    def _process_log_messages(self, node):
        msg = node.args[0]  # first arg to a logging method is the msg

        # if first arg is a call to a i18n name
        if (isinstance(msg, ast.Call)
                and isinstance(msg.func, ast.Name)
                and msg.func.id in self.i18n_names):
            self.add_error(msg, message=self.LOGGING_CHECK_DESC)

        # if the first arg is a reference to a i18n call
        elif (isinstance(msg, ast.Name)
                and msg.id in self.assignments):
            self.add_error(msg, message=self.LOGGING_CHECK_DESC)


def dict_constructor_with_sequence_copy(logical_line):
    """Should use a dict comprehension instead of a dict constructor.

    PEP-0274 introduced dict comprehension with performance enhancement
    and it also makes code more readable.

    Okay: lower_res = {k.lower(): v for k, v in res[1].items()}
    Okay: fool = dict(a='a', b='b')
    K008: lower_res = dict((k.lower(), v) for k, v in res[1].items())
    K008:     attrs = dict([(k, _from_json(v))
    K008: dict([[i,i] for i in range(3)])

    """
    MESSAGE = ("K008 Must use a dict comprehension instead of a dict"
               " constructor with a sequence of key-value pairs.")

    dict_constructor_with_sequence_re = (
        re.compile(r".*\bdict\((\[)?(\(|\[)(?!\{)"))

    if dict_constructor_with_sequence_re.match(logical_line):
        yield (0, MESSAGE)


def factory(register):
    register(CheckForMutableDefaultArgs)
    register(block_comments_begin_with_a_space)
    register(CheckForTranslationIssues)
    register(dict_constructor_with_sequence_copy)



# Copyright 2018 Catalyst Cloud Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
import uuid

from oslo_utils import timeutils

import freezegun
from keystone.common import provider_api
from keystone.common import utils
import keystone.conf
from keystone import exception
from keystone.models import receipt_model
from keystone import receipt
from keystone.tests import unit
from keystone.tests.unit import ksfixtures
from keystone.tests.unit.ksfixtures import database


CONF = keystone.conf.CONF
PROVIDERS = provider_api.ProviderAPIs

DELTA = datetime.timedelta(seconds=CONF.receipt.expiration)
CURRENT_DATE = timeutils.utcnow()


class TestReceiptProvider(unit.TestCase):
    def setUp(self):
        super(TestReceiptProvider, self).setUp()
        self.useFixture(database.Database())
        self.useFixture(
            ksfixtures.KeyRepository(
                self.config_fixture,
                'fernet_receipts',
                CONF.fernet_receipts.max_active_keys
            )
        )
        self.load_backends()

    def test_unsupported_receipt_provider(self):
        self.config_fixture.config(group='receipt',
                                   provider='MyProvider')
        self.assertRaises(ImportError,
                          receipt.provider.Manager)

    def test_provider_receipt_expiration_validation(self):
        receipt = receipt_model.ReceiptModel()
        receipt.issued_at = utils.isotime(CURRENT_DATE)
        receipt.expires_at = utils.isotime(CURRENT_DATE - DELTA)
        receipt.id = uuid.uuid4().hex
        with freezegun.freeze_time(CURRENT_DATE):
            self.assertRaises(exception.ReceiptNotFound,
                              PROVIDERS.receipt_provider_api._is_valid_receipt,
                              receipt)

        # confirm a non-expired receipt doesn't throw errors.
        # returning None, rather than throwing an error is correct.
        receipt = receipt_model.ReceiptModel()
        receipt.issued_at = utils.isotime(CURRENT_DATE)
        receipt.expires_at = utils.isotime(CURRENT_DATE + DELTA)
        receipt.id = uuid.uuid4().hex
        with freezegun.freeze_time(CURRENT_DATE):
            self.assertIsNone(
                PROVIDERS.receipt_provider_api._is_valid_receipt(receipt))

    def test_validate_v3_none_receipt_raises_receipt_not_found(self):
        self.assertRaises(
            exception.ReceiptNotFound,
            PROVIDERS.receipt_provider_api.validate_receipt,
            None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import keystone.conf


CONF = keystone.conf.CONF


def symptom_unreasonable_max_token_size():
    """`keystone.conf [DEFAULT] max_token_size` should be adjusted.

    This option is intended to protect keystone from unreasonably sized tokens,
    where "reasonable" is mostly dependent on the `keystone.conf [token]
    provider` that you're using. If you're using one of the following token
    providers, then you should set `keystone.conf [DEFAULT] max_token_size`
    accordingly:

    - For Fernet, set `keystone.conf [DEFAULT] max_token_size = 255`, because
      Fernet tokens should never exceed this length in most deployments.
      However, if you are also using `keystone.conf [identity] driver = ldap`,
      Fernet tokens may not be built using an efficient packing method,
      depending on the IDs returned from LDAP, resulting in longer Fernet
      tokens (adjust your `max_token_size` accordingly).
    """
    return ('fernet' in CONF.token.provider and CONF.max_token_size > 255)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import itertools

from keystone.common.policies import access_rule
from keystone.common.policies import access_token
from keystone.common.policies import application_credential
from keystone.common.policies import auth
from keystone.common.policies import base
from keystone.common.policies import consumer
from keystone.common.policies import credential
from keystone.common.policies import domain
from keystone.common.policies import domain_config
from keystone.common.policies import ec2_credential
from keystone.common.policies import endpoint
from keystone.common.policies import endpoint_group
from keystone.common.policies import grant
from keystone.common.policies import group
from keystone.common.policies import identity_provider
from keystone.common.policies import implied_role
from keystone.common.policies import limit
from keystone.common.policies import mapping
from keystone.common.policies import policy
from keystone.common.policies import policy_association
from keystone.common.policies import project
from keystone.common.policies import project_endpoint
from keystone.common.policies import protocol
from keystone.common.policies import region
from keystone.common.policies import registered_limit
from keystone.common.policies import revoke_event
from keystone.common.policies import role
from keystone.common.policies import role_assignment
from keystone.common.policies import service
from keystone.common.policies import service_provider
from keystone.common.policies import token
from keystone.common.policies import token_revocation
from keystone.common.policies import trust
from keystone.common.policies import user


def list_rules():
    return itertools.chain(
        base.list_rules(),
        access_rule.list_rules(),
        access_token.list_rules(),
        application_credential.list_rules(),
        auth.list_rules(),
        consumer.list_rules(),
        credential.list_rules(),
        domain.list_rules(),
        domain_config.list_rules(),
        ec2_credential.list_rules(),
        endpoint.list_rules(),
        endpoint_group.list_rules(),
        grant.list_rules(),
        group.list_rules(),
        identity_provider.list_rules(),
        implied_role.list_rules(),
        limit.list_rules(),
        mapping.list_rules(),
        policy.list_rules(),
        policy_association.list_rules(),
        project.list_rules(),
        project_endpoint.list_rules(),
        protocol.list_rules(),
        region.list_rules(),
        registered_limit.list_rules(),
        revoke_event.list_rules(),
        role.list_rules(),
        role_assignment.list_rules(),
        service.list_rules(),
        service_provider.list_rules(),
        token_revocation.list_rules(),
        token.list_rules(),
        trust.list_rules(),
        user.list_rules(),
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import datetime
import mock
import uuid

import freezegun
from oslo_db import exception as oslo_db_exception
from oslo_utils import timeutils
import six
from six.moves import http_client
from testtools import matchers

from keystone.common import provider_api
from keystone.common import utils
from keystone.models import revoke_model
from keystone.tests.unit import test_v3

PROVIDERS = provider_api.ProviderAPIs


def _future_time_string():
    expire_delta = datetime.timedelta(seconds=1000)
    future_time = timeutils.utcnow() + expire_delta
    return utils.isotime(future_time)


class OSRevokeTests(test_v3.RestfulTestCase, test_v3.JsonHomeTestMixin):

    JSON_HOME_DATA = {
        'https://docs.openstack.org/api/openstack-identity/3/ext/OS-REVOKE/1.0'
        '/rel/events': {
            'href': '/OS-REVOKE/events',
        },
    }

    def test_get_empty_list(self):
        resp = self.get('/OS-REVOKE/events')
        self.assertEqual([], resp.json_body['events'])

    def _blank_event(self):
        return {}

    # The two values will be the same with the exception of
    # 'issued_before' which is set when the event is recorded.
    def assertReportedEventMatchesRecorded(self, event, sample, before_time):
        after_time = timeutils.utcnow()
        event_issued_before = timeutils.normalize_time(
            timeutils.parse_isotime(event['issued_before']))
        self.assertLessEqual(
            before_time, event_issued_before,
            'invalid event issued_before time; %s is not later than %s.' % (
                utils.isotime(event_issued_before, subsecond=True),
                utils.isotime(before_time, subsecond=True)))
        self.assertLessEqual(
            event_issued_before, after_time,
            'invalid event issued_before time; %s is not earlier than %s.' % (
                utils.isotime(event_issued_before, subsecond=True),
                utils.isotime(after_time, subsecond=True)))
        del (event['issued_before'])
        del (event['revoked_at'])
        self.assertEqual(sample, event)

    def test_revoked_list_self_url(self):
        revoked_list_url = '/OS-REVOKE/events'
        resp = self.get(revoked_list_url)
        links = resp.json_body['links']
        self.assertThat(links['self'], matchers.EndsWith(revoked_list_url))

    def test_revoked_token_in_list(self):
        audit_id = uuid.uuid4().hex
        sample = self._blank_event()
        sample['audit_id'] = six.text_type(audit_id)
        before_time = timeutils.utcnow().replace(microsecond=0)
        PROVIDERS.revoke_api.revoke_by_audit_id(audit_id)
        resp = self.get('/OS-REVOKE/events')
        events = resp.json_body['events']
        self.assertEqual(1, len(events))
        self.assertReportedEventMatchesRecorded(events[0], sample, before_time)

    def test_disabled_project_in_list(self):
        project_id = uuid.uuid4().hex
        sample = dict()
        sample['project_id'] = six.text_type(project_id)
        before_time = timeutils.utcnow().replace(microsecond=0)
        PROVIDERS.revoke_api.revoke(
            revoke_model.RevokeEvent(project_id=project_id))

        resp = self.get('/OS-REVOKE/events')
        events = resp.json_body['events']
        self.assertEqual(1, len(events))
        self.assertReportedEventMatchesRecorded(events[0], sample, before_time)

    def test_disabled_domain_in_list(self):
        domain_id = uuid.uuid4().hex
        sample = dict()
        sample['domain_id'] = six.text_type(domain_id)
        before_time = timeutils.utcnow().replace(microsecond=0)
        PROVIDERS.revoke_api.revoke(
            revoke_model.RevokeEvent(domain_id=domain_id))

        resp = self.get('/OS-REVOKE/events')
        events = resp.json_body['events']
        self.assertEqual(1, len(events))
        self.assertReportedEventMatchesRecorded(events[0], sample, before_time)

    def test_list_since_invalid(self):
        self.get('/OS-REVOKE/events?since=blah',
                 expected_status=http_client.BAD_REQUEST)

    def test_list_since_valid(self):
        resp = self.get('/OS-REVOKE/events?since=2013-02-27T18:30:59.999999Z')
        events = resp.json_body['events']
        self.assertEqual(0, len(events))

    def test_since_future_time_no_events(self):
        domain_id = uuid.uuid4().hex
        sample = dict()
        sample['domain_id'] = six.text_type(domain_id)

        PROVIDERS.revoke_api.revoke(
            revoke_model.RevokeEvent(domain_id=domain_id))

        resp = self.get('/OS-REVOKE/events')
        events = resp.json_body['events']
        self.assertEqual(1, len(events))

        resp = self.get('/OS-REVOKE/events?since=%s' % _future_time_string())
        events = resp.json_body['events']
        self.assertEqual([], events)

    def test_revoked_at_in_list(self):
        time = datetime.datetime.utcnow()
        with freezegun.freeze_time(time) as frozen_datetime:
            revoked_at = timeutils.utcnow()
            # Given or not, `revoked_at` will always be set in the backend.
            PROVIDERS.revoke_api.revoke(
                revoke_model.RevokeEvent(revoked_at=revoked_at))

            frozen_datetime.tick(delta=datetime.timedelta(seconds=1))

            resp = self.get('/OS-REVOKE/events')
            events = resp.json_body['events']
            self.assertThat(events, matchers.HasLength(1))
            # Strip off the microseconds from `revoked_at`.
            self.assertTimestampEqual(utils.isotime(revoked_at),
                                      events[0]['revoked_at'])

    def test_access_token_id_not_in_event(self):
        ref = {'description': uuid.uuid4().hex}
        resp = self.post('/OS-OAUTH1/consumers', body={'consumer': ref})
        consumer_id = resp.result['consumer']['id']
        PROVIDERS.oauth_api.delete_consumer(consumer_id)

        resp = self.get('/OS-REVOKE/events')
        events = resp.json_body['events']
        self.assertThat(events, matchers.HasLength(1))
        event = events[0]
        self.assertEqual(consumer_id, event['OS-OAUTH1:consumer_id'])
        # `OS-OAUTH1:access_token_id` is None and won't be returned to
        # end user.
        self.assertNotIn('OS-OAUTH1:access_token_id', event)

    def test_retries_on_deadlock(self):
        patcher = mock.patch('sqlalchemy.orm.query.Query.delete',
                             autospec=True)

        # NOTE(mnikolaenko): raise 2 deadlocks and back to normal work of
        # method. Two attempts is enough to check that retry decorator works.
        # Otherwise it will take very much time to pass this test
        class FakeDeadlock(object):
            def __init__(self, mock_patcher):
                self.deadlock_count = 2
                self.mock_patcher = mock_patcher
                self.patched = True

            def __call__(self, *args, **kwargs):
                if self.deadlock_count > 1:
                    self.deadlock_count -= 1
                else:
                    self.mock_patcher.stop()
                    self.patched = False
                raise oslo_db_exception.DBDeadlock

        sql_delete_mock = patcher.start()
        side_effect = FakeDeadlock(patcher)
        sql_delete_mock.side_effect = side_effect

        try:
            PROVIDERS.revoke_api.revoke(revoke_model.RevokeEvent(
                user_id=uuid.uuid4().hex))
        finally:
            if side_effect.patched:
                patcher.stop()

        call_count = sql_delete_mock.call_count

        # initial attempt + 1 retry
        revoke_attempt_count = 2
        self.assertEqual(call_count, revoke_attempt_count)
